/******************************************************************************
	Copyright (C) 2016-2024 DistroAV <contact@distroav.org>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, see <https://www.gnu.org/licenses/>.
******************************************************************************/

#pragma once

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#define LOG_VERBOSE 500

extern int LOG_LEVEL;

extern const char *PLUGIN_NAME;
extern const char *PLUGIN_DISPLAY_NAME;
extern const char *PLUGIN_VERSION;

void obs_log(int log_level, const char *format, ...);
// The following is from the obs-plugingtemplate, but when added here it create linkage error at build on windows. This is set in plugin-support.c.in instead.
// extern void blogva(int log_level, const char *format, va_list args);

#ifdef __cplusplus
}
#endif
