set -e
#/Applications/OBS.app/Contents/MacOS/OBS --distroav-debug --distroav-update-local --distroav-update-force --distroav-update-last-check-ignore
#/Applications/OBS.app/Contents/MacOS/OBS --obs-ndi-debug --obs-ndi-update-local --obs-ndi-update-force --obs-ndi-update-last-check-ignore
/Applications/OBS.app/Contents/MacOS/OBS ${@}
